
#pragma rtGlobals=1		// Use modern global access method.
#pragma IgorVersion=5
#pragma version = 1	 // Last Modified: 2013/07/24 by Jamie Boyd
#pragma module = "GUIP"
//Some math utility functions, probably more to follow

	
//******************************************************************************************************
// Returns  the least common multiple of all the values in a wave. Adapted from http://www.cut-the-knot.org/Curriculum/Arithmetic/LCM.shtml
//Algorithm for Computing the LCM
//Let there be a finite sequence of positive integers X = (x1, x2, ..., xn), n > 1. The algorithm proceeds in steps as follows: 
// on each step m it examines and updates the sequence X(m) = (x1(m), x2(m), ..., xn(m)), X(1) = X. The purpose of the examination is 
//to pick the least (perhaps, one of many) element of the sequence X(m). Assuming xk0(m) is the selected element, the sequence X(m+1) is defined as
//
//xk(m+1) = xk(m), k  k0
//
//xk0(m+1)= xk0(m) + xk0.
//
//In other words, the least element is increased by the corresponding x whereas the rest of the elements pass from X(m) to X(m+1) unchanged.
//The algorithm stops when all elements in sequence X(m) are equal. Their common value L is exactly LCM(X).
function GUIPLCM_wave (theWave)
	Wave theWave
	
	if (!(dataFolderExists ("root:packages")))
		newDataFolder root:packages
	endif
	//need a duplicate of the wave, make it in the packages folder
	duplicate/o theWave root:packages:LCMtempWave
	WAVE LCMWave = root:packages:LCMtempWave
	variable ii, nPnts = numpnts (theWave), doAnother 
	for (;;) // loops until it finds a solution, so empty initialization
		wavestats/q/M=1 LCMWave
		for (doAnother =0, ii =0; ii < nPnts; ii += 1)
			if (LCMWave [ii] == V_min) // loop through instead of just changing LCMWAve [V_minLoc] as multiple elements may be equal to V_min
				LCMWave [ii]  +=  theWave (V_minLoc)
			endif
			if (LCMWave [ii] != LCMWave [ii-1]) //do Another iteration if not all elements of the wave are equal
				doAnother =1
				for (;ii < nPnts; ii+=1) // we go into a sub loop to avoid checking the doAnother condition once it has been met
					if (LCMWave [ii] == V_min)
						LCMWave [ii]  += theWave (V_minLoc)
					endif
				endfor
				continue
			endif
		endfor
		if (!(doAnother))
			return LCMWave [0]
		endif
	endfor
end

//******************************************************************************************************
// reurns greatest common divisor for all values in the wave by pairwise computation
Function GUIPGCD_Wave (theWave)
	wave thewave
	
	variable ii, pnts = numpnts (theWave)
	variable vGCD = gcd(theWave [0], theWave [1])
	for (ii =2; ii < pnts; ii+=1)
		vGCD = GCD (vGCD,  theWave [ii])
	endfor
	return vGCD
end

//******************************************************************************************************
// this function calculates a  a signed 32 bit integer fractional equivalent for a number (input) given in decimal format.
// Variables to get the numerator and denominator of the fraction are passed by reference
// The general principle is to start with input = numerator, denominator =1, multiply both by a big constant, round numerator to an integer, and reduce
// the biggest signed 32 bit integer is 2^31 -1 = approx 2.14748e+09, so make sure neither numerator nor denominator will be bigger than that
// the chosen constant is a multiple of some comon numbers so many fractions will reduce nicely, and is a little bit less than 2^31
// RatioFromNumber operation was introduced  In Igor 6, obsoleting this function
STATIC CONSTANT kdec2FracMult =  1297296000 // 5*8*9*10*11*12*13*14*15
STATIC CONSTANT kOverFlow64=3471528 // biggest 64 bit int, 2^52-1/kdec2FracMult
Function  GUIPdec2frac (inPut, numerator, denominator)
	variable input
	variable &numerator, &denominator
	
	if (input ==0)
		numerator =0
		denominator = 1
	else
		variable mult=kdec2FracMult
		// can't overflow bigggest 64 float int, 2^52-1
		if (abs(inPut) > kOverFlow64)
			mult = floor (input/kOverFlow64)
		endif
		numerator = round (input * mult)
		denominator = mult
		// use GCD function to reduce fraction
		variable GreatestCommonDivisor = gcd (numerator, denominator)
		numerator/= GreatestCommonDivisor
		denominator /= GreatestCommonDivisor
		// output can't be bigger than 32bit int
		if (abs (numerator) > 2^31)
			mult = floor (mult/input)
			numerator = round (input * mult)
			denominator = mult
			// use GCD function to reduce fraction
		 	GreatestCommonDivisor = gcd (numerator, denominator)
			numerator/= GreatestCommonDivisor
			denominator /= GreatestCommonDivisor
		endif
	endif
	return 0
end

//******************************************************************************************************
// Function to test the dec2frac function
Function GUIPtestdec2frac (input)
	variable input
	
	variable numerator, denominator
	 GUIPdec2frac (inPut,numerator, denominator)
	 printf  "numerator =%d, denominator = %d\r"numerator, denominator
	 printf "Division Result = %.12f\r" numerator/denominator
	 printf "Error = %.12f%\r", 100 * (input - (numerator/denominator))/input
end

//******************************************************************************************************
// Binary Searches for sorted text waves. Not exactly math utilities, but sorting is mathematical
// THESE ASSUME THAT THE TEXT WAVE IS ALREADY ALPHABETICALLY SORTED
//******************************************************************************************************
// A utility function to calculate base 2 logarithms
STATIC CONSTANT log2 =  0.301029995664
Function GUIPlogBase2 (theValue)
	variable theValue
	
	return log (theValue)/log2
end

//******************************************************************************************************
// finds the first occurrence of a text within a range of points in a SORTED text wave
// returns the point number of the last occurrence, or -1 times (1 + the position before which where thetext should be inserted) if the text was not found
// the +1 offfset for missing values is so that a value found at pos 0 can be discriminated from a missing value that needs to be inserted before pos 0 
// Last Modified Mar 06 2012 by Jamie Boyd
function GUIPMathFirstText (theWave, thetext, startPos, endPos)
	wave/T theWave // alpabetically sorted text wave. 
	string thetext // the text to find in this wave, matching the entire contents of the wave at the returned point
	variable startPos, endPos // range over which to search. Can pass INF to seach whole wave
	
	// limit start and end positions to possible values
	startPos = max (0, startPos)
	endPos = min (numPnts (theWave)-1, endPos)
	if (endPos == -1)
		return -1
	endif
	variable iPos // the point to be compared
	variable  theCmp // the result of the comparison
	variable firstPt, lastPt// variables the define the range oer which comparisons will be made
	for (firstPt =startPos, lastPt = endPos; firstPt < lastPt; )
		iPos = trunc ((firstPt + lastPt)/2)
		theCmp = cmpStr (thetext, theWave [iPos])
		if (theCmp == 1) //thetext is alphabetically after theWave [iPos]
			firstPt = min (lastPt, iPos +1)
		elseif (theCmp ==-1)// thetext is alphabetically before theWave [iPos]
			lastPt =max (firstPt, iPos -1)
		else //thetext is the same as theWave [iPos]
			 if ((iPos ==startPos) || (cmpStr (theText, theWave [iPos -1]) == 1)) // then iPos is the first occurence of thetext in theWave from startPos to endPos
				return iPos
			else //  there are more copies of theText in theWave before iPos 
				lastPt = max (firstPt, iPos-1)
			endif
		endif
	endfor
	// when we exit the loop, firstPt and lastPt are the same. Either we are at the correct point, or point is not in this wave
	if (cmpStr (thetext, theWave [firstPt]) == 0)
		return firstPt 
	else
		if (cmpStr (theText, theWave [firstPt]) == 1)
			firstPt +=1
		endif
		return -(firstPt +1)
	endif
end

// searches through a specific row or a sepecific column in a 2D text wave SORTED on the specified row or column
function GUIPMathFirstText2D (theWave, thetext, startPos, endPos, dimNumber, rowOrcol)
	wave/T theWave // alpabetically sorted text wave. 
	string thetext // the text to find in this wave, matching the entire contents of the wave at the returned point
	variable startPos, endPos // range over which to search. Can pass INF to seach whole wave
	variable dimNumber // the dimension over which to search (0 to search a particular row, 1 to search a particular column)
	variable  rowOrCol // the row or column to search
	
	// limit start and end positions to possible values
	startPos = max (0, startPos)
	endPos = min (endPos, DimSize(theWave, dimNumber)-1)
	if (endPos == -1)
		return -1
	endif
	variable iPos // the point to be compared
	variable  theCmp // the result of the comparison
	variable firstPt, lastPt// variables the define the range oer which comparisons will be made
	for (firstPt =startPos, lastPt = endPos; firstPt < lastPt; )
		iPos = trunc ((firstPt + lastPt)/2)
		if (dimNumber == 0)
			theCmp = cmpStr (thetext, theWave [iPos] [roworCol])
		else
			theCmp = cmpStr (thetext, theWave [roworCol] [iPos] )
		endif
		if (theCmp == 1) //thetext is alphabetically after theWave [iPos]
			firstPt = min (lastPt, iPos +1)
		elseif (theCmp ==-1)// thetext is alphabetically before theWave [iPos]
			lastPt =max (firstPt, iPos -1)
		else //thetext is the same as theWave [iPos]
			if (dimNumber ==0)
				theCmp = (cmpStr (theText, theWave [iPos -1] [rowOrCol]))
			else
				theCmp = (cmpStr (theText, theWave  [rowOrCol] [iPos -1]))
			endif
			 if ((iPos ==startPos) || theCmp) // then iPos is the first occurence of thetext in theWave from startPos to endPos
				return iPos
			else //  there are more copies of theText in theWave before iPos 
				lastPt = max (firstPt, iPos-1)
			endif
		endif
	endfor
	// when we exit the loop, firstPt and lastPt are the same. Either we are at the correct point, or point is not in this wave
	if (dimNumber ==0)
		theCmp = cmpStr (thetext, theWave [firstPt] [rowOrCol])
	else
		theCmp = cmpStr (thetext, theWave [rowOrCol] [firstPt])
	endif
	if (theCmp == 0)
		return firstPt 
	else
		if (dimNumber ==0)
			theCmp = cmpStr (theText, theWave [firstPt] [rowOrCol])
		else
			theCmp = cmpStr (theText, theWave [rowOrCol] [firstPt])
		endif
		if (theCmp == 1)
			firstPt +=1
		endif
		return -(firstPt +1)
	endif
end


//******************************************************************************************************
// test for the findFirstText function's ability to return correct position for inserting a string that was not found in a text wave
//When this function ends, root:testFindFirstText should contain howMany strings of  random capital letters of length howLong, sorted alphabetically
// Last Modified Mar 06 2012 by Jamie Boyd
function GUIPtestFindFirstText (howMany, howLong)
	variable howMany
	variable howlong
	
	make/t/o/n = 0 root:testFindFirstText
	WAVE/t test = root:testFindFirstText
	variable iM, iL, pos
	string entry
	for (iM =0;iM < howMany; iM +=1)
		for (iL=0, entry=""; iL <  howLong; iL +=1)
			entry += num2Char (65 + floor (13 + enoise (13))) // the entry string will consist of random capital letters A-Z
		endfor
		pos= GUIPMathFirstText (test, entry, 0, inf)
		if (pos < 0) // text was not found
			// account for the 1 character offset in position used to disambiguate "-0 = not found, aphabetically before pos 0" and
			// "0 = found at position 0"
			pos = -(pos +1) 
		endif
		insertpoints pos, 1, test
		test [pos] = entry
	endfor
end

//******************************************************************************************************
// finds the last occurence of a text within a range of points in a SORTED text wave
// returns the point number of the last occurrence, or -1 times (1 + the position before which where thetext should be inserted) if the text was not found
// the +1 offset for missing value is so that a value found at pos 0 can be discriminated from a missing value that needs to be inserted before pos 0 
// Last Modified Mar 06 2012 by Jamie Boyd
function GUIPMathLastText(theWave, thetext, startPos, endPos)
	wave/T theWave // alpabetically sorted text wave. 
	string thetext // the text to find in this wave, matching the entire contents of the wave at that point
	variable startPos, endPos // range over which to search. 
	
	// limit start and end positions to possible values
	startPos = max (0, startPos)
	endPos = min (numPnts (theWave)-1, endPos)
	if (endPos == -1)
		return -1
	endif
	variable iPos // the point to be compared
	variable  theCmp // the result of the comparison
	variable firstPt, lastPt // variables the define the range over which comparisons will be made
	for (firstPt = startPos , lastPt = endPos; firstPt < lastPt; )
		iPos = trunc ((firstPt + lastPt)/2)
		theCmp = cmpStr (thetext, theWave [iPos])
		if (theCmp == 1) //thetext is alphabetically after theWave [iPos]
			firstPt = min (lastPt, iPos +1)
		elseif (theCmp == -1) // thetext is alphabetically before theWave [iPos]
			lastPt =max (firstPt, iPos -1)
		else //thetext is the same as theWave [iPos]
			 if ((iPos ==endPos) || (cmpStr (theText, theWave [iPos +1]) == -1)) // then iPos is the last occurence of thetext in theWave from startPos to endPos
				return iPos
			else //  there are more copies of theText in theWave after iPos 
				firstPt = min (lastPt, iPos +1)
			endif
		endif
	endfor
	// when we exit the loop, firstPt and lastPt are the same. Either we are at the correct point, or point is not in this wave
	if (cmpStr (thetext, theWave [firstPt]) == 0)
		return firstPt 
	else
		if (cmpStr (theText, theWave [firstPt]) == 1)
			firstPt +=1
		endif
		return -(firstPt +1)
	endif
end

//******************************************************************************************************
// the text versions were so useful, I've expanded to numeric
function GUIPMathFirstNum (theWave, theNum, startPos, endPos)
	wave theWave // sorted wave. 
	variable theNum // the number to find in this wave
	variable startPos, endPos // range over which to search. Can pass INF to seach whole wave
	
	// limit start and end positions to possible values
	startPos = max (0, startPos)
	endPos = min (numPnts (theWave)-1, endPos)
	if (endPos == -1)
		return -1
	endif
	variable iPos // the point to be compared
	variable  theCmp // the result of the comparison
	variable firstPt, lastPt// variables the define the range oer which comparisons will be made
	for (firstPt =startPos, lastPt = endPos; firstPt < lastPt; )
		iPos = trunc ((firstPt + lastPt)/2)
		//theCmp =  cmpStr (thetext, theWave [iPos])
		if (theNum > theWave [iPos] ) //thetext is alphabetically after theWave [iPos]
			firstPt = min (lastPt, iPos +1)
		elseif (theNum < theWave [iPos] )// thetext is alphabetically before theWave [iPos]
			lastPt =max (firstPt, iPos -1)
		else //thetext is the same as theWave [iPos]
			 if ((iPos ==startPos) || (theNum > theWave [iPos -1])) 
			// if ((iPos ==startPos) || (cmpStr (theText, theWave [iPos -1]) == 1)) // then iPos is the first occurence of thetext in theWave from startPos to endPos
				return iPos
			else //  there are more copies of theText in theWave before iPos 
				lastPt = max (firstPt, iPos-1)
			endif
		endif
	endfor
	// when we exit the loop, firstPt and lastPt are the same. Either we are at the correct point, or point is not in this wave
	if (theNum == theWave [firstPt]) 
	//if (cmpStr (thetext, theWave [firstPt]) == 0)
		return firstPt 
	else
		if (theNum > theWave [firstPt])
		//if (cmpStr (theText, theWave [firstPt]) == 1)
			firstPt +=1
		endif
		return -(firstPt +1)
	endif
end

function GUIPMathLastNum(theWave, theNum, startPos, endPos)
	wave theWave // alpabetically sorted  wave. 
	variable theNum // the value to find in this wave
	variable startPos, endPos // range over which to search. 
	
	// limit start and end positions to possible values
	startPos = max (0, startPos)
	endPos = min (numPnts (theWave)-1, endPos)
	if (endPos == -1)
		return -1
	endif
	variable iPos // the point to be compared
	variable  theCmp // the result of the comparison
	variable firstPt, lastPt // variables the define the range over which comparisons will be made
	for (firstPt = startPos , lastPt = endPos; firstPt < lastPt; )
		iPos = trunc ((firstPt + lastPt)/2)
		//theCmp = cmpStr (thetext, theWave [iPos])
		if (theNum > theWave [iPos]) //thetext is alphabetically after theWave [iPos]
			firstPt = min (lastPt, iPos +1)
		elseif (theNum < theWave [iPos]) // thetext is alphabetically before theWave [iPos]
			lastPt =max (firstPt, iPos -1)
		else //thetext is the same as theWave [iPos]
			 if ((iPos ==endPos) || (theNum < theWave [iPos +1])) // then iPos is the last occurence of thetext in theWave from startPos to endPos
				return iPos
			else //  there are more copies of theText in theWave after iPos 
				firstPt = min (lastPt, iPos +1)
			endif
		endif
	endfor
	// when we exit the loop, firstPt and lastPt are the same. Either we are at the correct point, or point is not in this wave
	if (theNum == theWave [firstPt])
		return firstPt 
	else
		if (theNum > theWave [firstPt])
			firstPt +=1
		endif
		return -(firstPt +1)
	endif
end

//******************************************************************************************************
// A test of the GUIPMathFirstText and GUIPMathLastText functions, using FindValue as a baseline
// both accuracy and speed are tested
// Last Modified Feb 03 2012 by Jamie Boyd

function GUIPtestFindFirstLastText (maxN)
	variable maxN
	
	variable iPt, nPts = floor (GUIPlogBase2 (maxN))
	// output waves
	make/o/n=(nPts) FindValueFirstTime_out, FindValueLastTime_out, FFTOtime_out, FLTOtime_out
	setScale d 0,0 ,"s" FindValueFirstTime_out, FindValueLastTime_out, FFTOtime_out, FLTOtime_out
	make/o/n=(nPts) FindValueFirstPos_out, FindValueLastPos_out, FFTOpos_out, FLTOpos_out
	// make data wave
	variable iN, iVal, valToN
	make/o/T/n =(maxN) testTextWave
	WAVE/T testTextWave
	for (iN =0, iVal =0;iN < maxN; iVal +=1)
		for (valToN = iN + ceil (GUIPlogBase2 (iVal +2)); iN < valToN;  iN +=1)
			testTextWave [iN] = num2str (iVal)
		endfor
	endfor
	// sort it, because CmpStr does not do alphanumeric comparison properly, 10 is befor 9
	Sort testTextWave, testTextWave
	// varables for finding and timing
	string toFind
	variable fPos, myTimer, elapsedTime
	for (iPt =0; iPt < nPts; ipt += 1)
		// find increasingly further values in the wave
		 toFind = testTextWave [2^ (iPt + 2)]
		// find first occurrence
		// use find value
		myTimer = startmstimer
		FindValue/S=0 /TEXT=toFind/TXOP=4  testTextWave
		elapsedTime = stopMSTimer(myTimer )/1e06
		FindValueFirstTime_out [iPt] = elapsedTime
		FindValueFirstPos_out [iPt] = V_Value
		// use GUIPMathFirstText
		myTimer = startmstimer
		fPos = GUIPMathFirstText (testTextWave, toFind, 0, maxN)
		elapsedTime = stopMSTimer(myTimer )
		FFTOtime_out [iPt] = elapsedTime/1e06
		FFTOpos_out  [iPt]  = fPos
		// find last occurrence
		// with findValue followed by a loop
		myTimer = startmstimer
		FindValue/S=0 /TEXT=toFind/TXOP=4  testTextWave
		for (iN= V_Value +1;cmpStr (testTextWave [iN], toFind) ==0 && iN < maxN; iN +=1)
		endfor
		elapsedTime = stopMSTimer(myTimer )/1e06
		FindValueLastTime_out [iPt] = elapsedTime
		FindValueLastPos_out [iPt] = iN-1
		// with GUIPMathLastText
		myTimer = startmstimer
		fPos = GUIPMathLastText (testTextWave, toFind, 0, maxN)
		elapsedTime = stopMSTimer(myTimer )
		FLTOtime_out [iPt] = elapsedTime/1e06
		FLTOpos_out  [iPt]  = fPos
	endfor
	display FFTOtime_out, FLTOtime_out vs FindValueFirstPos_out
	modifygraph rgb = (0,0,0)
	appendtoGraph FindValueFirstTime_out, FindValueLastTime_out vs FindValueLastPos_out
	modifygraph mode =4
	label bottom "Points in Text Wave"
	label left "Time to Find Element (\\U)"
	Legend/C/N=text0/F=0/B=1/A=MT
	ModifyGraph log(bottom)=1
	edit FindValueFirstPos_out, FFTOpos_out, FindValueLastPos_out, FLTOpos_out
end

//*******************************************************************************************************************************************
// gets rgb values from a color table, used to color traces in a logical order, with nColors spread evenly over the colorTable
// returns 1 if requested colorTable does not exist, else 0 for success
// Last Modified Jan 31 3012 by Jamie Boyd
function GUIPcolorRamp (cTable, iColor, nColors, rVal, gVal, bVal)
	string cTable // name of an Igor color table. One with a regular progression of colors (Rainbow, Rainbow256, RainbowCycle, etc.) works well
	variable iColor // the number of the current trace to color
	variable nColors // the total number of traces to be colored (or the number at which the cycle of traces should repeat)
	variable &rVal // pass-by-reference variable to hold red value of returned color
	variable &gVal // pass-by-reference variable to hold green value of returned color
	variable &bVal // pass-by-reference variable to hold blue value of returned color
	
	string cTableClean = cleanupName (cTable, 1) 
	wave/z cTableWave = $"root:packages:" + cTableClean
	if(!(waveExists (cTableWave)))
		if (WhichListItem(cTable, CTabList(), ";", 0,0) ==-1)
			print "An Igor color table names \"" + cTable + "\" does not exist."
			return 1
		endif
		if (!(dataFolderExists ("root:Packages")))
			newdatafolder root:packages
		endif
		string savedFldr = getdatafolder (1)
		setdatafolder root:packages
		ColorTab2Wave $cTable
		Rename M_Colors, $cTableClean
		setdatafolder $savedFldr
		wave cTableWave = $"root:packages:" + cTableClean
	endif
	variable pos = mod (iColor, nColors) * dimsize (cTableWave, 0)/(nColors -1)
	// use mod so color table wraps around if iColor > nColors, as for circular data with a circular colorTable
	rVal = cTableWave [pos] [0]
	gval = cTableWave [pos] [1]
	bval = cTableWave [pos] [2]
	return 0
end

//******************************************************************************************************
//******************** Gaussian kernels, 1D and 2D *************************************************************
//******************************************************************************************************
// Returns a 1D Gaussian using Pascal's Triangle.  Using this method, the kernel width scales with the  log of nK
// Last Modified 2013/07/30
Function GaussianLinePT (nK)
	variable nK //kernel width
	
	make/o/n = (nK) kernelPT
	kernelPT = 0
	kernelPT [0] = 1/2^(nk -1)
	variable ii, ij
	for (ii =1; ii < nK; ii += 1)
		for (ij =nK-1 ; ij > 0; iJ -=1)
			kernelPT [ij] += kernelPT [ij-1]
		endfor
	endfor
	return nK
end

//******************************************************************************************************
// method to make a Gaussian kernel with a defined sigma
// Last Modified 2013/07/30
function GaussianLine1 (GausRadius, KernelRadii)
	variable GausRadius // Radius of Gaussian (i.e., sigma). Full Width at half height will be sigma * sqrt (2)
	variable KernelRadii // how many sigma to include in the kernel. 1, 2, or 3 suggested. Kernel width will be GausRadius * kernelRadii * 2
		
	variable iPnt, nK = GausRadius * kernelRadii * 2
	if (mod (round (nk), 2) == 0) 
		if (round (nK) < nk) // fractional part of nK is less than 0.5
			nk = round (nk) + 1
		else //  fractional part of nK is greater than 0.5
			nk = round (nk) - 1
		endif
	else
		nk = round (nk)
	endif
	make/o/n=(nK) kernel1
	variable sumG
	for (iPnt =0, sumG=0; iPnt < nK; iPnt +=1)
		kernel1 [iPnt] =  e^-((iPnt - kernelRadii * GausRadius)^2/(2 * GausRadius^2))
		sumG += kernel1 [iPnt] 
	endfor
	kernel1/= sumG
end

//******************************************************************************************************
// method to make a Gaussian kernel where kernel width and cut-off factor for kernel truncation are both specified. 
// Gaussian width of the filter (in pixels)  is specified as the sigma of the Guassian distriution.
// Sigma is equal to full width at half height / sqrt(2)
// Cut Off Factor defines how many sigmas of the Gaussian to clip the kernel size to. This is converted to a decimal value
// for the cut off using the erf function:
// x Sigma = erf (x/sqrt(2))			cutoff = 1-  erf (x/sqrt(2))
// 
// 1 sigma= 0.682689492137086		cutoff= 0.3173105078629140
//1.5 sigma=0.866385597462284	cutoff=0.1336144025377159
// 2 sigma= 0.954499736103642		cutoff=0.0455002638963580
// 2.5 sigma = 0.987580669348448	cutoff=0.0124193306515520
// 3 sigma = 0.997300203936740	cutoff=0.0026997960632600
// 3.5 sigma =0.999534741841929	cutoff=0.0004652581580710
// 4 sigma = 0.999936657516334	cutoff=0.0000633424836660
// 4.5 sigma =0.999993204653751	cutoff=0.0000067953462490
// 5 sigma = 0.999999426696856	cutoff=0.0000005733031440
// 5.5 sigma = 0.999999962020875	cutoff=0.0000000379791250
// 6 sigma = 0.999999998026825	cutoff=0.0000000019731750
// 6.5 sigma = 0.999999999919680	cutoff=0.0000000000803200
// 7 sigma = 0.999999999997440	cutoff=0.0000000000025600

// The number of pixels in the kernel is 1 + (1+ 2 * cutOffFactSigma) * Gaussian sigma (in pixels)
// Last Modified 2013/08/02
Function  GaussianLine2 (kernelSigma, cutOffFactSigma)
	variable kernelSigma		// sigma of the Gaussian, in pixels
	variable cutOffFactSigma  //  How many sigmas of the kernel to show. 1.5 to 3 is a reasonable range
	
	// calculate pixel width of the kernel with given Gaussian width and cutoff
	variable nk = (1 + (1+ 2 * cutOffFactSigma) * kernelSigma)
	//  round kernel width to a whole number
	// if width is even, make it odd by adding 1 if rounding down
	// subtracting 1 if rounding up
	if (mod (round (nk), 2) == 0) 
		if (round (nK) < nk) // fractional part of nK is less than 0.5
			nk = round (nk) + 1
		else //  fractional part of nK is greater than 0.5
			nk = round (nk) - 1
		endif
	else
		nk = round (nk)
	endif
	// calculate cutoff factor (proportion) from cutOff factor (sigmas) using error function
	variable cutOffFact = 1 -  erf (cutOffFactSigma/sqrt(2))
	// make and fill the kernel
	make/o/n = (nK) kernel2
	kernel2 = e^((ln(cutOffFact) * (2*p - (nK-1))^2)/(nK-1)^2)
	// return width of kernel
	return nk
end
